<kr-sidebar-nav>
  <ul>
    <li>
      <a href="/backend/index/">
        <span class="material-icons material-icons-outlined">home</span>
        Index
      </a>
    </li>
    <li>
      <a href="/backend/form/">
        <span class="material-icons material-icons-outlined">dynamic_form</span>
        Form test
      </a>
    </li>
    <li>
      <a href="/backend/table/">
        <span class="material-icons material-icons-outlined">grid_on</span>
        Table
        <span class="material-icons material-icons-outlined expand">expand_more</span>
      </a>
      <ul class="collapse">
        <li><a href="/backend/table/filtered/">Filtered table</a></li>
      </ul>
    </li>
    <li>
      <a b:href="djsamon.url('backend:charts')">
        <span class="material-icons material-icons-outlined">bar_chart</span>
        Charts
      </a>
    </li>
    <!-- li>
      <a href="/backend/index/">
        <span class="material-icons material-icons-outlined">person_outline</span>
        Partnerek
        <span class="material-icons material-icons-outlined expand">expand_more</span>
      </a>
      <ul class="collapse">
        <li><a href="#cimkek">Címkék</a></li>
      </ul>
    </li>
    <li>
      <a href="#">
        <span class="material-icons material-icons-outlined">content_copy</span>
        Dokumentumok
      </a>
    </li -->
  </ul>
</kr-sidebar-nav>