<kr-side-modal width="800px">
    <kr-modal-header>
        <h5>Title text 2</h5>
    </kr-modal-header>

    <kr-modal-body>
        <kr-form id="modalForm" b:object="form" action="/backend/modal/" label-width="sm-4 md-3 lg-4" field-width="sm-8 md-9 lg-8" />
    </kr-modal-body>

    <kr-modal-footer>
        <input class="btn btn-primary" type="button" value="Mentés" name="save" @click="${ (ev) => this.submit(ev) }" />
    </kr-modal-footer>
</kr-side-modal>
