<head>
    <meta charset="utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no" />
    <title>kirUI</title>

    <script type="text/javascript" b:src="djsamon.static('django_kirui/tinymce/tinymce.min.js')"></script>
    <script type="text/javascript" b:src="djsamon.static('jquery/js/jquery-3.6.0.min.js')"></script>
    <link rel="stylesheet" b:href="djsamon.bundled_static('bootstrap/scss/bootstrap.scss.css')" />
    <link href="https://fonts.googleapis.com/css2?family=Material+Icons" rel="stylesheet" />
</head>